@@ -54,7 +54,9 @@ else:
 try:
     from unittest import mock  # py3
 except ImportError:
-    import mock  # NOQA - requires "pip install mock"
+    with warnings.catch_warnings():
+        warnings.simplefilter("ignore")
+        import mock  # NOQA - requires "pip install mock"
 
 if sys.version_info >= (3, 4):
     import enum
@@ -70,7 +72,7 @@ __all__ = [
     'VERBOSITY',
     "HAS_CPU_AFFINITY", "HAS_CPU_FREQ", "HAS_ENVIRON", "HAS_PROC_IO_COUNTERS",
     "HAS_IONICE", "HAS_MEMORY_MAPS", "HAS_PROC_CPU_NUM", "HAS_RLIMIT",
-    "HAS_SENSORS_BATTERY", "HAS_BATTERY""HAS_SENSORS_FANS",
+    "HAS_SENSORS_BATTERY", "HAS_BATTERY", "HAS_SENSORS_FANS",
     "HAS_SENSORS_TEMPERATURES", "HAS_MEMORY_FULL_INFO",
     # subprocesses
     'pyrun', 'reap_children', 'get_test_subprocess', 'create_zombie_proc',
@@ -123,7 +125,7 @@ NO_RETRIES = 10
 # bytes tolerance for system-wide memory related tests
 MEMORY_TOLERANCE = 500 * 1024  # 500KB
 # the timeout used in functions which have to wait
-GLOBAL_TIMEOUT = 3
+GLOBAL_TIMEOUT = 3 if TRAVIS or APPVEYOR else 0.5
 # test output verbosity
 VERBOSITY = 1 if os.getenv('SILENT') or TOX else 2
 # be more tolerant if we're on travis / appveyor in order to avoid
@@ -160,7 +162,7 @@ HAS_PROC_CPU_NUM = hasattr(psutil.Process, "cpu_num")
 HAS_RLIMIT = hasattr(psutil.Process, "rlimit")
 HAS_THREADS = hasattr(psutil.Process, "threads")
 HAS_SENSORS_BATTERY = hasattr(psutil, "sensors_battery")
-HAS_BATTERY = HAS_SENSORS_BATTERY and psutil.sensors_battery()
+HAS_BATTERY = HAS_SENSORS_BATTERY and bool(psutil.sensors_battery())
 HAS_SENSORS_FANS = hasattr(psutil, "sensors_fans")
 HAS_SENSORS_TEMPERATURES = hasattr(psutil, "sensors_temperatures")
 
@@ -168,13 +170,28 @@ HAS_SENSORS_TEMPERATURES = hasattr(psutil, "sensors_temperatures")
 
 
 def _get_py_exe():
-    exe = os.path.realpath(sys.executable)
-    if not os.path.exists(exe):
-        # It seems this only occurs on OSX.
-        exe = which("python%s.%s" % sys.version_info[:2])
-        if not exe or not os.path.exists(exe):
-            ValueError("can't find python exe real abspath")
-    return exe
+    def attempt(exe):
+        try:
+            subprocess.check_call(
+                [exe, "-V"], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+        except Exception:
+            return None
+        else:
+            return exe
+
+    if OSX:
+        exe = \
+            attempt(sys.executable) or \
+            attempt(os.path.realpath(sys.executable)) or \
+            attempt(which("python%s.%s" % sys.version_info[:2])) or \
+            attempt(psutil.Process().exe())
+        if not exe:
+            raise ValueError("can't find python exe real abspath")
+        return exe
+    else:
+        exe = os.path.realpath(sys.executable)
+        assert os.path.exists(exe), exe
+        return exe
 
 
 PYTHON_EXE = _get_py_exe()
@@ -193,7 +210,11 @@ _testfiles_created = set()
 def _cleanup_files():
     DEVNULL.close()
     for name in os.listdir(u('.')):
-        if name.startswith(u(TESTFILE_PREFIX)):
+        if isinstance(name, unicode):
+            prefix = u(TESTFILE_PREFIX)
+        else:
+            prefix = TESTFILE_PREFIX
+        if name.startswith(prefix):
             try:
                 safe_rmpath(name)
             except Exception:
@@ -264,7 +285,7 @@ class ThreadTask(threading.Thread):
 # ===================================================================
 
 
-def _cleanup_on_err(fun):
+def _reap_children_on_err(fun):
     @functools.wraps(fun)
     def wrapper(*args, **kwargs):
         try:
@@ -275,7 +296,7 @@ def _cleanup_on_err(fun):
     return wrapper
 
 
-@_cleanup_on_err
+@_reap_children_on_err
 def get_test_subprocess(cmd=None, **kwds):
     """Creates a python subprocess which does nothing for 60 secs and
     return it as subprocess.Popen instance.
@@ -308,7 +329,7 @@ def get_test_subprocess(cmd=None, **kwds):
     return sproc
 
 
-@_cleanup_on_err
+@_reap_children_on_err
 def create_proc_children_pair():
     """Create a subprocess which creates another one as in:
     A (us) -> B (child) -> C (grandchild).
@@ -336,7 +357,7 @@ def create_proc_children_pair():
         subp = pyrun(s)
     child1 = psutil.Process(subp.pid)
     data = wait_for_file(_TESTFN2, delete=False, empty=False)
-    os.remove(_TESTFN2)
+    safe_rmpath(_TESTFN2)
     child2_pid = int(data)
     _pids_started.add(child2_pid)
     child2 = psutil.Process(child2_pid)
@@ -380,7 +401,7 @@ def create_zombie_proc():
             conn.close()
 
 
-@_cleanup_on_err
+@_reap_children_on_err
 def pyrun(src, **kwds):
     """Run python 'src' code string in a separate interpreter.
     Returns a subprocess.Popen instance.
@@ -397,7 +418,7 @@ def pyrun(src, **kwds):
     return subp
 
 
-@_cleanup_on_err
+@_reap_children_on_err
 def sh(cmd, **kwds):
     """run cmd in a subprocess and return its output.
     raises RuntimeError on error.
@@ -461,7 +482,9 @@ def reap_children(recursive=False):
         try:
             subp.terminate()
         except OSError as err:
-            if err.errno != errno.ESRCH:
+            if WINDOWS and err.errno == 6:  # "invalid handle"
+                pass
+            elif err.errno != errno.ESRCH:
                 raise
         if subp.stdout:
             subp.stdout.close()
@@ -640,7 +663,7 @@ def wait_for_file(fname, delete=True, empty=False):
     if not empty:
         assert data
     if delete:
-        os.remove(fname)
+        safe_rmpath(fname)
     return data
 
 
@@ -662,12 +685,34 @@ def call_until(fun, expr):
 
 def safe_rmpath(path):
     "Convenience function for removing temporary test files or dirs"
+    def retry_fun(fun):
+        # On Windows it could happen that the file or directory has
+        # open handles or references preventing the delete operation
+        # to succeed immediately, so we retry for a while. See:
+        # https://bugs.python.org/issue33240
+        stop_at = time.time() + 1
+        while time.time() < stop_at:
+            try:
+                return fun()
+            except WindowsError as _:
+                err = _
+                if err.errno != errno.ENOENT:
+                    raise
+                else:
+                    warn("ignoring %s" % (str(err)))
+                    time.sleep(0.01)
+        raise err
+
     try:
         st = os.stat(path)
         if stat.S_ISDIR(st.st_mode):
-            os.rmdir(path)
+            fun = functools.partial(shutil.rmtree, path)
         else:
-            os.remove(path)
+            fun = functools.partial(os.remove, path)
+        if POSIX:
+            fun()
+        else:
+            retry_fun(fun)
     except OSError as err:
         if err.errno != errno.ENOENT:
             raise
@@ -753,16 +798,21 @@ unittest.TestCase = TestCase
 
 
 def _setup_tests():
-    os.environ['PSUTIL_TESTING'] = '1'
-    assert psutil._psplatform.cext.py_psutil_testing()
+    if 'PSUTIL_TESTING' not in os.environ:
+        # This won't work on Windows but set_testing() below will do it.
+        os.environ['PSUTIL_TESTING'] = '1'
+    psutil._psplatform.cext.set_testing()
 
 
 def get_suite():
-    testmodules = [os.path.splitext(x)[0] for x in os.listdir(HERE)
-                   if x.endswith('.py') and x.startswith('test_') and not
-                   x.startswith('test_memory_leaks')]
+    testmods = [os.path.splitext(x)[0] for x in os.listdir(HERE)
+                if x.endswith('.py') and x.startswith('test_') and not
+                x.startswith('test_memory_leaks')]
+    if "WHEELHOUSE_UPLOADER_USERNAME" in os.environ:
+        testmods = [x for x in testmods if not x.endswith((
+                    "osx", "posix", "linux"))]
     suite = unittest.TestSuite()
-    for tm in testmodules:
+    for tm in testmods:
         # ...so that the full test paths are printed on screen
         tm = "psutil.tests.%s" % tm
         suite.addTest(unittest.defaultTestLoader.loadTestsFromName(tm))
